use std::env;
use std::fs::read_to_string;
use std::process::Command;
use tempfile::NamedTempFile;
use std::io::{self, Write};

const ENV_MPS_EDITOR: &str = "MPS_EDITOR";
const DEFAULT_EDITOR: &str = "nano";


// Use with content "" (empty string) for adding new item
pub fn open_to_edit(content: &String) -> io::Result<String> {
    // Create a temporary file
    let mut temp_file = NamedTempFile::new()?;

    // Write content
    write!(temp_file, "{}", content)?;

    // Get the user's preferred editor from the $EDITOR environment variable
    // Default is 'nano'
    let editor = env::var(ENV_MPS_EDITOR).unwrap_or_else(|_| DEFAULT_EDITOR.to_string());

    // Get the path of the temp file
    let file_path = temp_file.path().to_str().unwrap().to_string();

    // Open the file in the external editor
    Command::new(editor)
        .arg(&file_path)
        .status()
        .map_err(|e| io::Error::new(
            io::ErrorKind::Other,
            format!("Failed to launch editor: {}", e)
        ))?;

    // Read the file content after editing
    let mut edited_content = read_to_string(&file_path)?;

    // Remove only one trailing newline if it exists
    // because editor like vim or nano adds to the end new line
    if edited_content.ends_with('\n') {
        edited_content.pop();
    }

    // Print the edited content
    Ok(edited_content)
}

pub fn open_to_show(content: &String) -> io::Result<()> {
    let mut temp_file = NamedTempFile::new()?;

    // Write the content to the file
    write!(temp_file, "{}", content)?;

    // Default is 'nano'
    let editor = env::var(ENV_MPS_EDITOR).unwrap_or_else(|_| DEFAULT_EDITOR.to_string());

    let file_path = temp_file.path().to_str().unwrap().to_string();

    // Open the file in the external editor
    Command::new(editor)
        .arg(&file_path)
        .status()
        .map_err(|e| io::Error::new(
            io::ErrorKind::Other,
            format!("Failed to launch editor: {}", e)
        ))?;
    Ok(())
}

